/* This thread does all the work. It communicates with the client through Envelopes.
 * 
 */
import java.lang.Thread;
import java.net.Socket;
import java.io.*;
import java.util.*;

public class GroupThread extends Thread 
{
	private final Socket socket;
	private GroupServer my_gs;
	
	public GroupThread(Socket _socket, GroupServer _gs)
	{
		socket = _socket;
		my_gs = _gs;
	}
	
	public void run()
	{
		boolean proceed = true;

		try
		{
			//Announces connection and opens object streams
			System.out.println("*** New connection from " + socket.getInetAddress() + ":" + socket.getPort() + "***");
			final ObjectInputStream input = new ObjectInputStream(socket.getInputStream());
			final ObjectOutputStream output = new ObjectOutputStream(socket.getOutputStream());
			
			do
			{
				Envelope message = (Envelope)input.readObject();
				System.out.println("Request received: " + message.getMessage());
				Envelope response;
				
				if(message.getMessage().equals("GET"))//Client wants a token
				{
					String username = (String)message.getObjContents().get(0); //Get the username
					if(username == null)
					{
						response = new Envelope("FAIL");
						response.addObject(null);
						output.writeObject(response);
					}
					else
					{
						UserToken yourToken = createToken(username); //Create a token
						
						//Respond to the client. On error, the client will receive a null token
						response = new Envelope("OK");
						response.addObject(yourToken);
						output.writeObject(response);
					}
				}
				else if(message.getMessage().equals("CUSER")) //Client wants to create a user
				{
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								String username = (String)message.getObjContents().get(0); //Extract the username
								UserToken yourToken = (UserToken)message.getObjContents().get(1); //Extract the token
								
								if(createUser(username, yourToken))
								{
									response = new Envelope("OK"); //Success
								}
							}
						}
					}
					
					output.writeObject(response);
				}
				else if(message.getMessage().equals("DUSER")) //Client wants to delete a user
				{
					
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								String username = (String)message.getObjContents().get(0); //Extract the username
								UserToken yourToken = (UserToken)message.getObjContents().get(1); //Extract the token
								
								if(deleteUser(username, yourToken))
								{
									response = new Envelope("OK"); //Success
								}
							}
						}
					}
					
					output.writeObject(response);
				}
				else if(message.getMessage().equals("CGROUP")) //Client wants to create a group
				{
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								String groupname = (String)message.getObjContents().get(0); //Extract group name
								UserToken yourToken = (UserToken)message.getObjContents().get(1); //Extract token
								
								if(createGroup(groupname, yourToken))
								{
									response = new Envelope("OK"); //Success
								}
							}
						}
					}
					
					output.writeObject(response);
				}
				else if(message.getMessage().equals("DGROUP")) //Client wants to delete a group
				{
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								String groupname = (String)message.getObjContents().get(0); //Extract group name
								UserToken yourToken = (UserToken)message.getObjContents().get(1); //Extract token
								
								if(deleteGroup(groupname, yourToken))
								{
									response = new Envelope("OK"); //Success
								}
							}
						}
					}
					
					output.writeObject(response);
				}
				else if(message.getMessage().equals("LMEMBERS")) //Client wants a list of members in a group
				{
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								String groupname = (String)message.getObjContents().get(0); //Extract groupname
								UserToken yourToken = (UserToken)message.getObjContents().get(1); //Extract token
								
								List<String> theMembers = listMembers(groupname, yourToken); //List to be returned to the client
								
								if(theMembers == null) //No list returned
								{
									response = new Envelope("FAIL"); //Failure
									response.addObject(null);
								}
								else
								{
									response = new Envelope("OK"); //Success
									response.addObject(theMembers);
								}
							}
						}
					}
					
					output.writeObject(response);
				}
				else if(message.getMessage().equals("AUSERTOGROUP")) //Client wants to add user to a group
				{
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								if(message.getObjContents().get(2) != null)
								{
									String username = (String)message.getObjContents().get(0); //Extract username 
									String groupname = (String)message.getObjContents().get(1); //Extract groupname
									UserToken yourToken = (UserToken)message.getObjContents().get(2); //Extract token
									
									if(addUserToGroup(username, groupname, yourToken))
									{
										response = new Envelope("OK"); //Success
									}
								}
							}
						}
					}
						
					output.writeObject(response);
				}
				else if(message.getMessage().equals("RUSERFROMGROUP")) //Client wants to remove user from a group
				{
					if(message.getObjContents().size() < 2)
					{
						response = new Envelope("FAIL");
					}
					else
					{
						response = new Envelope("FAIL");
						
						if(message.getObjContents().get(0) != null)
						{
							if(message.getObjContents().get(1) != null)
							{
								if(message.getObjContents().get(2) != null)
								{
									String username = (String)message.getObjContents().get(0); //Extract username
									String groupname = (String)message.getObjContents().get(1); //Extract groupname
									UserToken yourToken = (UserToken)message.getObjContents().get(2); //Extract token
									
									if(removeUserFromGroup(username, groupname, yourToken))
									{
										response = new Envelope("OK"); //Success
									}
								}
							}
						}
					}
					
					output.writeObject(response);
				}
				else if(message.getMessage().equals("DISCONNECT")) //Client wants to disconnect
				{
					socket.close(); //Close the socket
					proceed = false; //End this communication loop
				}
				else
				{
					response = new Envelope("FAIL"); //Server does not understand client request
					output.writeObject(response);
				}
			}while(proceed);	
		}
		catch(Exception e)
		{
			System.err.println("Error: " + e.getMessage());
			e.printStackTrace(System.err);
		}
	}
	
	//Method to create tokens
	private UserToken createToken(String username) 
	{
		//Check that user exists
		if(my_gs.userList.checkUser(username))
		{
			//Issue a new token with server's name, user's name, and user's groups
			UserToken yourToken = new Token(my_gs.name, username, my_gs.userList.getUserGroups(username));
			return yourToken;
		}
		else
		{
			return null;
		}
	}
	
	//Method to create a group
	private boolean createGroup(String groupname, UserToken yourToken)
	{
		String requester = yourToken.getSubject();
		
		//Check that requester exists
		if(my_gs.userList.checkUser(requester))
		{
			//Does group already exist?
			if(my_gs.groupList.checkGroup(groupname))
			{
				return false;
			}
			else
			{
				//Add group to the list
				my_gs.groupList.addGroup(requester, groupname);
				
				//Add group to user and indicate they are an owner
				my_gs.userList.addGroup(requester, groupname);
				my_gs.userList.addOwnership(requester, groupname);
				return true;
			}
		}
		else
		{
			
			return false;
		}
	}
	
	//Method to delete a group
	private boolean deleteGroup(String groupname, UserToken yourToken)
	{
		String requester = yourToken.getSubject();
		//Does requester exist?
		if(my_gs.userList.checkUser(requester))
		{
			//Is the requester an owner?
			if(my_gs.userList.getUserOwnership(requester).contains(groupname))
			{
				
				//Get membership list from the group list
				List<String> deleteFromUsers = my_gs.groupList.getMembers(groupname);
				
				//Go into each user and delete the reference to the group
				for(int index = 0; index < deleteFromUsers.size(); index++)
				{
					my_gs.userList.removeGroup(deleteFromUsers.get(index), groupname);
				}
				
				//Remove requester's ownership of this group
				my_gs.userList.removeOwnership(requester, groupname);
				
				//Remove group from the group list
				my_gs.groupList.deleteGroup(groupname);
				
				return true;
			}
			else
			{
			
				return false; //Requester not an owner
				
			}
		}		
		else
		{
			
			return false; //Requester does not exist
		}
	}
	
	//Method to create a user
	private boolean createUser(String username, UserToken yourToken)
	{
		String requester = yourToken.getSubject();
		
		//Check if requester exists
		if(my_gs.userList.checkUser(requester))
		{
			//Get the user's groups
			ArrayList<String> temp = my_gs.userList.getUserGroups(requester);
			//requester needs to be an administrator
			if(temp.contains("ADMIN"))
			{
				//Does user already exist?
				if(my_gs.userList.checkUser(username))
				{
					return false; //User already exists
				}
				else
				{
					my_gs.userList.addUser(username);
					return true;
				}
			}
			else
			{
				return false; //requester not an administrator
			}
		}
		else
		{
			return false; //requester does not exist
		}
	}
	
	//Method to delete a user
	private boolean deleteUser(String username, UserToken yourToken)
	{
		String requester = yourToken.getSubject();
		
		//Does requester exist?
		if(my_gs.userList.checkUser(requester))
		{
			ArrayList<String> temp = my_gs.userList.getUserGroups(requester);
			//requester needs to be an administer
			if(temp.contains("ADMIN"))
			{
				//Does user exist?
				if(my_gs.userList.checkUser(username))
				{
					//User needs deleted from the groups they belong
					ArrayList<String> deleteFromGroups = new ArrayList<String>();
					
					//This will produce a hard copy of the list of groups this user belongs
					for(int index = 0; index < my_gs.userList.getUserGroups(username).size(); index++)
					{
						deleteFromGroups.add(my_gs.userList.getUserGroups(username).get(index));
					}
					
					//Delete the user from the groups
					//If user is the owner, removeMember will automatically delete group!
					for(int index = 0; index < deleteFromGroups.size(); index++)
					{
						my_gs.groupList.removeMember(username, deleteFromGroups.get(index));
					}
					
					//If groups are owned, they must be deleted
					ArrayList<String> deleteOwnedGroup = new ArrayList<String>();
					
					//Make a hard copy of the user's ownership list
					for(int index = 0; index < my_gs.userList.getUserOwnership(username).size(); index++)
					{
						deleteOwnedGroup.add(my_gs.userList.getUserOwnership(username).get(index));
					}
					
					//Delete owned groups
					for(int index = 0; index < deleteOwnedGroup.size(); index++)
					{
						//Use the delete group method. Token must be created for this action
						deleteGroup(deleteOwnedGroup.get(index), new Token(my_gs.name, username, deleteOwnedGroup));
					}
					
					//Delete the user from the user list
					my_gs.userList.deleteUser(username);
					
					return true;	
				}
				else
				{
					return false; //User does not exist
					
				}
			}
			else
			{
				return false; //requester is not an administer
			}
		}
		else
		{
			return false; //requester does not exist
		}
	}
	
	private List<String> listMembers(String groupname, UserToken yourToken)
	{
		String requester = yourToken.getSubject();
		
		//Does requester exist?
		if(my_gs.userList.checkUser(requester))
		{
			ArrayList<String> temp = my_gs.userList.getUserOwnership(requester);
			//requester must be an owner of the group
			if(temp.contains(groupname))
			{
					//New arraylist must be assembled. If I return the list directly, errors occur
					ArrayList<String> brandNewList = new ArrayList<String>();
					
					//Put the members of the group in the new list
					for(String member: my_gs.groupList.getMembers(groupname))
					{
						brandNewList.add(member);
					}
					
					return brandNewList;
			}
			else
			{
				return null; //Requester is not an owner or group does not exist
			}
		}
		else
		{
			return null; //Requester does not exist
		}
		
	}
	
	private boolean addUserToGroup(String username, String groupname, UserToken token)
	{
		String requester = token.getSubject();
		
		//Requester must exist
		if(my_gs.userList.checkUser(requester))
		{
			ArrayList<String> temp = my_gs.userList.getUserOwnership(requester);
			//Is requester an owner of the group?
			if(temp.contains(groupname))
			{
				//Does user exist?
				if(my_gs.userList.checkUser(username))
				{
					//Is user already a member?
					ArrayList<String> memberTemp = my_gs.groupList.getMembers(groupname);
					if(memberTemp.contains(username))
					{
						return false; //User already member
					}
					else
					{
						my_gs.userList.addGroup(username, groupname); //Add group to the user
						my_gs.groupList.addMember(username, groupname); //Add user to group
						return true;
					}
				}
				else
				{
					return false; //User does not exist
				}
			}
			else
			{
				return false; //Requester is not an owner
			}
		}
		else
		{	
			return false; //Requester does not exist
		}
	}
	
	//Method to remove user from a group
	private boolean removeUserFromGroup(String username, String groupname, UserToken token)
	{
		String requester = token.getSubject();
	
		//Does the requester exist?
		if(my_gs.userList.checkUser(requester))
		{
			ArrayList<String> temp = my_gs.userList.getUserOwnership(requester);
			//Is requester an owner of the group?
			if(temp.contains(groupname))
			{
				//Does user exist?
				if(my_gs.userList.checkUser(username))
				{
					//Is the user in the group?
					ArrayList<String> memberTemp = my_gs.groupList.getMembers(groupname);
					if(memberTemp.contains(username))
					{
						//Is user the owner? If yes, delete group!
						if(my_gs.groupList.getOwner(groupname).equals(username))
						{
							deleteGroup(groupname, new Token(my_gs.name, username, my_gs.userList.getUserGroups(username)));
							return true;
						}
						else
						{
							my_gs.userList.removeGroup(username, groupname); //Remove group from user
							my_gs.groupList.removeMember(username, groupname); //Remove user from group
							return true;
						}
					}
					else
					{
						return false; //User not in group
					}
				}
				else
				{
					return false; //User does not exist
				}
			}
			else
			{
				return false; //Requester is not an owner
			}
		}
		else
		{
			return false; //Requester does not exist
		}
	}
}
