/* Driver program for FileSharing client */

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;

public class RunClient {
	public static void main(String[] args) throws IOException {
		InputStreamReader ISR = new InputStreamReader(System.in);
        BufferedReader BR = new BufferedReader(ISR);
		
		GroupClient groupClient = new GroupClient();
		FileClient fileClient = new FileClient();
		UserToken myToken = null;
		String username = ""; /*Default user is empty string*/
		String groupServerAddress = "localhost";
		String fileServerAddress = "localhost";
		int groupServerPort = 8765;
		int fileServerPort = 4321;
				
		String input = "" ;
		
		while (input.compareTo("quit") != 0 && 
			   input.compareTo("q") != 0) {							//  menu command line
			 System.out.printf("FileSharing> ") ;
        	 input = BR.readLine();	
        	 if (input == null) break;
			 String[] chunks = input.split(" ") ;
			 if (chunks.length < 1) {
				 /*  empty command - do nothing */
			 }
			 else if (chunks[0].compareTo("a")==0 ||
				 chunks[0].compareTo("addr")==0 ||
				 chunks[0].compareTo("address")==0) {
				 /* command is a request to change a server address */
				 if (chunks.length < 3) {
					 System.out.printf("Usage:  > address [-g|-f] [server address]\n");					 
				 }
				 else if (chunks[1].compareTo("-g")==0) {
					 groupServerAddress = chunks[2];
					 System.out.printf("Group server address changed to %s\n", groupServerAddress);						 
				 }
				 else if (chunks[1].compareTo("-f")==0) {
					 fileServerAddress = chunks[2];
					 System.out.printf("File server address changed to %s\n", fileServerAddress);					
				 }
			 }
			 else if (chunks[0].compareTo("p")==0 ||
					 chunks[0].compareTo("port")==0) {
				 /* command is a request to change a server port */
				 if (chunks.length < 3) {
					 System.out.printf("Usage:  > port [-g|-f] [server port]\n");					 
				 }
				 else if (Integer.parseInt(chunks[2]) <= 0) {
					 System.out.printf("Error: Invalid port number %d\n", Integer.parseInt(chunks[2]));					 
				 }
				 else if (chunks[1].compareTo("-g")==0) {
					 groupServerPort = Integer.parseInt(chunks[2]);
					 System.out.printf("Group server port changed to %d\n", Integer.parseInt(chunks[2]));						 
				 }
				 else if (chunks[1].compareTo("-f")==0) {
					 fileServerPort = Integer.parseInt(chunks[2]);
					 System.out.printf("File server port changed to %d\n", Integer.parseInt(chunks[2]));					
				 }				 
			 }
			 else if (chunks[0].compareTo("u")==0 ||
					 chunks[0].compareTo("user")==0) {
				 /* command is a request to change the username */
				 if (chunks.length < 2) {
					 System.out.printf("Usage:  > user [username]\n");					 
				 }
				 else {
					 username = chunks[1];
					 myToken = null; //Erase previous user's token
					 System.out.printf("Username changed to %s\n", chunks[1]);
				 }				 
			 }
			 else if (chunks[0].compareTo("h") == 0 ||
					 chunks[0].compareTo("help") == 0) {
				/* command is a request to see the command reference */
				 System.out.printf("%50s\n", "FileSharingClient Commands Reference") ;
				 System.out.printf("%-40s > (t)oken\n", "Obtain token from group server") ;
				 System.out.printf("%-40s > (u)ser [username]\n", "Change username") ;
				 System.out.printf("%-40s > (a)ddress [-g|-f] [server address]\n", "Change group or file server address") ;
				 System.out.printf("%-40s > (p)ort [-g|-f] [server port]\n", "Change group or file server port") ;
				 System.out.printf("%-40s > (d)isplay\n", "Display current settings") ;
				 System.out.printf("%-40s > (c)reate_(u)ser [username]\n", "Create a new user") ;
				 System.out.printf("%-40s > (d)elete_(u)ser [username]\n", "Delete a user") ;
				 System.out.printf("%-40s > (c)reate_(g)roup [groupname]\n", "Create a new group") ;
				 System.out.printf("%-40s > (d)elete_(g)roup [groupname]\n", "Delete a group") ;
				 System.out.printf("%-40s > (a)dd_(t)o_(g)roup [username] [groupname]\n", "Add a user to a group") ;
				 System.out.printf("%-40s > (d)elete_(f)rom_(g)roup [username] [groupname]\n", "Delete a user from a group") ;
				 System.out.printf("%-40s > (l)ist_(m)embers\n", "List members of a group") ;
				 System.out.printf("%-40s > (l)ist_(f)iles\n", "List files") ;
				 System.out.printf("%-40s > (d)ownload_(f)ile [remote path] [(optional) local path]\n", "Download a file");
				 System.out.printf("%-40s > (u)pload_(f)ile [group] [local path] [(optional) remote path]\n", "Upload a file") ;
				 System.out.printf("%-40s > (r)emove_(f)ile [remote path]\n", "Remove a file") ;
				 System.out.printf("%-40s > (h)elp\n", "Display commands reference") ;
				 System.out.printf("%-40s > (q)uit\n", "Exit FileSharingClient") ;
				 System.out.printf("Whereever you give a path, it must include the filename.\n");
				 
			 }
			else if (chunks[0].compareTo("d") == 0 ||
					 chunks[0].compareTo("display") == 0) {			
				/* command is a request to display current settings */
				System.out.printf("%30s\n", "Current settings") ;
				System.out.printf("%-40s%s\n", "Username:", username) ;
				System.out.printf("%-40s%s:%d\n", "Group server:", groupServerAddress, groupServerPort) ;
				System.out.printf("%-40s%s:%d\n", "File server:", fileServerAddress, fileServerPort) ;
				if (myToken == null) {
					System.out.printf("%-40s%s\n", "Token:", "<none>") ;
				}
				else {
					System.out.printf("%-40s%s\n", "Token groups:", myToken.getGroups().toString()) ;
					System.out.printf("%-40s%s\n", "Token issuer:", myToken.getIssuer()) ;
					System.out.printf("%-40s%s\n", "Token subject:", myToken.getSubject()) ;				
				}			
			}
			/* BEGIN FILE SERVER REQUESTS */
			 
			 else if (chunks[0].compareTo("t")==0 ||
					 chunks[0].compareTo("token")==0) {
				 /* command is a request to obtain a token from the group server */
				 
				if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) {
			 
								
					myToken = groupClient.getToken(username);
					if(myToken == null)
					{
						System.out.println("Error getting token");
					}
					else {
						System.out.println("Token issuer: " + myToken.getIssuer());
						System.out.println("Token subject: " + myToken.getSubject());
						System.out.println("Token groups: ");
						for(String group: myToken.getGroups()){
							System.out.println(group);
						}
					}
				}
				else {
					System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
				}
			}
			else if (chunks[0].compareTo("lf")==0 || chunks[0].compareTo("list_files")== 0) {
				 /* command is a request for a list of files */
				 if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (fileClient.isConnected() || fileClient.connect(fileServerAddress, fileServerPort)) 
					 {
						 List<String> fileList = fileClient.listFiles(myToken);
						 if(fileList != null)
						{
							System.out.println("Files:");
							for(String file: fileList)
							{
								System.out.println(file);
							}
						}
						else 
						{
							System.out.println("No files available.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + fileServerAddress + " on port " + fileServerPort);
					 }
				 }
			 }
			else if (chunks[0].compareTo("uf")==0 || chunks[0].compareTo("upload_file")== 0) {
				 /* command is to upload a file */
				
				 if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else if (chunks.length < 3) {
					 System.out.println("Usage: > (u)pload_(f)ile [group] [local path] [(optional) remote path]"); 
				 }
				 else if (!myToken.getGroups().contains(chunks[1])){
					 System.out.printf("Error: your token not valid for group %s\n", chunks[1]);
				 }
				 else {
					 String localPath, remotePath, group;
					 
					 group = chunks[1];
					 localPath = chunks[2];
					 File f = new File(localPath);
					 if (f.exists()) {

						 if (chunks.length>3) {
							 remotePath = chunks[3];
							 if (remotePath.charAt(0)!='/') {
								 remotePath = "/"+remotePath;
							 }
						 }
						 else {
							 chunks = localPath.split("/");
							 remotePath = "/"+chunks[chunks.length-1];
							 System.out.printf("No remote path specified.  Extracted filename %s\n", remotePath);
						 }
						 
						 if (fileClient.isConnected() || fileClient.connect(fileServerAddress, fileServerPort)) 
						 {
							 fileClient.upload(localPath, remotePath, group, myToken);
						 }	
						 else 
						 {
							 System.out.println("Error: couldn't connect to " + fileServerAddress + " on port " + fileServerPort);
						 }
					}
					 else {
						 System.out.printf("Error: file %s doesn't exist\n", localPath);
					 }
				 }
			 }
			else if (chunks[0].compareTo("df")==0 || chunks[0].compareTo("download_file")== 0) {
				 /* command is to download a file */
				
				 if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else if (chunks.length < 2) {
					 System.out.println("Usage: > (d)ownload_(f)ile [(optional) local path] [remote path]"); 
				 }
				 else {
					 String localPath, remotePath;
					 if (chunks.length == 2) {
						 remotePath = chunks[1];
						 localPath = chunks[1];
					 }
					 else {
						 localPath = chunks[2];
						 remotePath = chunks[1];
					 }
					 
					 if (localPath.charAt(0)=='/') {
						 localPath = localPath.substring(1);
					 }
				
					 if (fileClient.isConnected() || fileClient.connect(fileServerAddress, fileServerPort)) 
					 {
						 fileClient.download(remotePath, localPath, myToken);
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + fileServerAddress + " on port " + fileServerPort);
					 }
				 }
			 }
			 else if (chunks[0].compareTo("rf")==0 || chunks[0].compareTo("remove_file")== 0) 
			 {
				 /* command is to remove a file */
				
				 if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else if (chunks.length < 2) {
					 System.out.println("Usage: > (r)emove_(f)ile [remote path]"); 
				 }
				 else {
					 String remotePath = chunks[1];

					 if (fileClient.isConnected() || fileClient.connect(fileServerAddress, fileServerPort)) 
					 {
						 fileClient.delete(remotePath, myToken);
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + fileServerAddress + " on port " + fileServerPort);
					 }
				 }
			 }
			 /* END FILE SERVER REQUESTS */ 


			 /* BEGIN GROUP SERVER REQUESTS */ 
			 else if (chunks[0].compareTo("cu")==0 || chunks[0].compareTo("create_user")== 0) 
			 {
				 /* command is a request to create a new user on the group server */
				 if(chunks.length < 2) //user name not provided
				 {
					System.out.println("Usage: create_user [username]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 {
						if(groupClient.createUser(chunks[1], myToken))
						{
							System.out.println(chunks[1] + " created.");
						}
						else 
						{
							System.out.println("Error creating user. You may not be an administrator or user already exists.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 }
			 else if (chunks[0].compareTo("du")==0 || chunks[0].compareTo("delete_user")== 0) 
			 {
				 /* command is a request to delete a user on the group server */
				 if(chunks.length < 2) //user name not provided
				 {
					System.out.println("Usage: delete_user [username]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 {
						  
						if(groupClient.deleteUser(chunks[1], myToken))
						{
							System.out.println(chunks[1] + " deleted.");
						}
						else 
						{
							System.out.println("Error deleting user. You may not be an administrator or user does not exist.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 } 
			 else if (chunks[0].compareTo("cg")==0 || chunks[0].compareTo("create_group")== 0) 
			 {
				 /* command is a request to create a group on the group server */
				 if(chunks.length < 2) //user name not provided
				 {
					System.out.println("Usage: create_group [groupname]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 {
 
						if(groupClient.createGroup(chunks[1], myToken))
						{
							System.out.println(chunks[1] + " created.");
						}
						else 
						{
							System.out.println("Error creating group. Group may already exist.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 }
			 else if (chunks[0].compareTo("dg")==0 || chunks[0].compareTo("delete_group")== 0) 
			 {
				 /* command is a request to delete a group on the group server */
				 if(chunks.length < 2) //user name not provided
				 {
					System.out.println("Usage: delete_group [groupname]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 {
						  
						if(groupClient.deleteGroup(chunks[1], myToken))
						{
							System.out.println(chunks[1] + " deleted.");
						}
						else 
						{
							System.out.println("Error deleting group. You may not be the owner or group may not exist.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 }
			 else if (chunks[0].compareTo("atg")==0 || chunks[0].compareTo("add_to_group")== 0) 
			 {
				 /* command is a request to add a new user to a group on the group server */
				 if(chunks.length < 3) //user name not provided
				 {
					System.out.println("Usage: add_to_group [username] [groupname]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 {
						if(groupClient.addUserToGroup(chunks[1], chunks[2], myToken))
						{
							System.out.println(chunks[1] + " added to " + chunks[2]);
						}
						else 
						{
							System.out.println("Error adding user to group. You may not be the owner or group/user may not exist.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 }
			 else if (chunks[0].compareTo("dfg")==0 || chunks[0].compareTo("delete_from_group")== 0) 
			 {
				 /* command is a request to delete a user form a group on the group server */
				 if(chunks.length < 3) //user name not provided
				 {
					System.out.println("Usage: delete_from_group [username] [groupname]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 { 
						if(groupClient.deleteUserFromGroup(chunks[1], chunks[2], myToken))
						{
							System.out.println(chunks[1] + " deleted from " + chunks[2]);
						}
						else 
						{
							System.out.println("Error deleting user from group. You may not be the owner or group/user may not exist.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 }
			 else if (chunks[0].compareTo("lm")==0 || chunks[0].compareTo("list_members")== 0) 
			 {
				 /* command is a request to list members of a group on the group server */
				 if(chunks.length < 2) //user name not provided
				 {
					System.out.println("Usage: list_members [groupname]");
				 }
				 else if(myToken == null)
				 {
					 System.out.println("Error: no token");
				 }
				 else
				 {
					 if (groupClient.isConnected() || groupClient.connect(groupServerAddress, groupServerPort)) 
					 {
						 List<String> memberList = groupClient.listMembers(chunks[1], myToken);
						if(memberList != null)
						{
							System.out.println("Members of " + chunks[1]);
							for(String member: memberList)
							{
								System.out.println(member);
							}
						}
						else 
						{
							System.out.println("Error listing members of the group. You may not be the owner or user may not exist.");
						}
					 }	
					 else 
					 {
						 System.out.println("Error: couldn't connect to " + groupServerAddress + " on port " + groupServerPort);
					 }
				 }
			 }
			 /* END GROUP SERVER REQUESTS */
			 
		}// while
		
		// close any connections that were opened
		groupClient.disconnect();
		fileClient.disconnect();
		
	}// main
}// class
